package org.spagic.smx.components.registry;

import javax.annotation.PostConstruct;
import javax.jbi.JBIException;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.bean.support.TransformBeanSupport;
import org.apache.servicemix.jbi.FaultException;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.xpath.DefaultXPath;
import org.spagic.registry.utils.IServiceRegistry;
import org.spagic.registry.utils.ServiceRegistryFactory;
import javax.xml.registry.infomodel.Organization;
import javax.xml.registry.infomodel.Service;
import javax.xml.registry.infomodel.ServiceBinding;

import java.util.*;

public class ServiceRegistryInvoker extends TransformBeanSupport {
	
	private static final Log log = LogFactory.getLog(ServiceRegistryInvoker.class);
	private String serviceRegistryType;
	private String organizationXPath;
	private String serviceName;
	private String serviceURI;
	
	private IServiceRegistry serviceRegistry = null;
	
	private static final String QUERY_MANAGER_URL = "javax.xml.registry.queryManagerURL";
	private static final String LIFE_CICLE_MANAGER_URL = "javax.xml.registry.lifeCycleManagerURL";
	private static final String USERNAME = "registry.username";
	private static final String PASSWORD = "registry.password";
	

	@PostConstruct
	public void init() throws JBIException {
		log.debug("start method invoked...");
		Properties properties = new Properties();
		ResourceBundle resourceBundle = ResourceBundle.getBundle("spagic-serviceRegistry");
		properties.setProperty(QUERY_MANAGER_URL, resourceBundle.getString(getKeyPrefixProperty() + "inquiryUrl"));
		properties.setProperty(LIFE_CICLE_MANAGER_URL, resourceBundle.getString(getKeyPrefixProperty() + "publishUrl"));
		properties.setProperty(USERNAME, resourceBundle.getString(getKeyPrefixProperty() + "user"));
		properties.setProperty(PASSWORD, resourceBundle.getString(getKeyPrefixProperty() + "password"));

		
		if (getServiceRegistryType().equalsIgnoreCase("uddi")) {
			//leggere da smx TODO
			properties.setProperty("metadb.db.driver", "");
			properties.setProperty("metadb.db.user", "");
			properties.setProperty("metadb.db.password", "");
			properties.setProperty("metadb.db.url", "");
		}
		try{
			serviceRegistry = ServiceRegistryFactory.getServiceRegistry(serviceRegistryType, properties);
			serviceRegistry.initRegistry();
		}catch (Exception e) {
			e.printStackTrace();
		}catch (Throwable t){
			t.printStackTrace();
		}finally{
			serviceRegistry.finalizeRegistry();
		}
	}
	
	private String getKeyPrefixProperty() {
		return serviceRegistryType + "_";
	}
	protected boolean transform(MessageExchange exchange, NormalizedMessage in,
			NormalizedMessage out) throws Exception {
		try{	
			SourceTransformer sourceTransformer = new SourceTransformer();
	        String flussoXMLInput = sourceTransformer.contentToString(in); 
	        Document document = null;
			document = DocumentHelper.parseText(flussoXMLInput);
			DefaultXPath orgXPathExpression = new DefaultXPath(organizationXPath);
			String organizationName = orgXPathExpression.valueOf(document);
			
			Organization organitation = serviceRegistry.getOrganizationByName(organizationName);
			Service service = serviceRegistry.getServiceByOrganizationAndServiceName(organitation, serviceName);
				
			Collection serviceBindings = service.getServiceBindings();
			if(!serviceBindings.isEmpty()) {
				serviceURI = ((ServiceBinding)serviceBindings.iterator().next()).getAccessURI();
				if (!serviceURI.endsWith("/"))
					serviceURI = serviceURI.concat("/");
				out.setProperty("org.apache.servicemix.http.destination.uri", serviceURI);
			}
			out.setContent(exchange.getMessage("in").getContent());
		}catch(Throwable t) {
            log.error("Exception in ServiceRegistryInvoker");
            log.error(t);
            Fault fault = exchange.createFault();
            fault.setContent(new StringSource("<ERROR>An error is occured in service registry caller</ERROR>"));
            fault.setProperty("org.apache.servicemix.xml", in.getContent());
            throw new FaultException("ServiceRegistryInvoker Failed ", exchange, fault);
        }
		return true;
	}

	public String getServiceRegistryType() {
		return serviceRegistryType;
	}

	public void setServiceRegistryType(String serviceRegistryType) {
		this.serviceRegistryType = serviceRegistryType;
	}

	public String getOrganizatioXPath() {
		return organizationXPath;
	}

	public void setOrganizationXPath(String organitationXPath) {
		this.organizationXPath = organitationXPath;
	}

	public String getServiceName() {
		return serviceName;
	}

	public void setServiceName(String serviceName) {
		this.serviceName = serviceName;
	}
}
