package org.spagic.smx.components.resolver;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.bean.support.TransformBeanSupport;
import org.hibernate.Session;
import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.dbutils.HibernateUtil;
import org.spagic.metadb.model.Process;
import org.spagic.metadb.model.Step;


public class ServiceResolver extends TransformBeanSupport{

	private static final String NAMESPACE = "NAMESPACE".intern();
	private static final String LOCALPART = "LOCALPART".intern();
	private static final String PREFIX = "PREFIX".intern();
	private static final String STATIC = "Static".intern();
	private static final String SPECIFIC = "Specific".intern();
	
	private static final Log log = LogFactory.getLog(ServiceResolver.class);
	
	private String subProcess;
	private String binding;
	private String versionToUse;
	private String staticVersion;

	public String getSubProcess() {
		return subProcess;
	}

	public void setSubProcess(String subProcess) {
		this.subProcess = subProcess;
	}

	public String getBinding() {
		return binding;
	}

	public void setBinding(String binding) {
		this.binding = binding;
	}

	public String getVersionToUse() {
		return versionToUse;
	}

	public void setVersionToUse(String versionToUse) {
		this.versionToUse = versionToUse;
	}

	public String getStaticVersion() {
		return staticVersion;
	}

	public void setStaticVersion(String staticVersion) {
		this.staticVersion = staticVersion;
	}


	@Override
	protected boolean transform(MessageExchange exchange, NormalizedMessage in, NormalizedMessage out) throws Exception {
		
		String namespace= null, localpart= null, prefix="SpagicResolver";
		
		Session aSession = null;
		Process process = null;
		try {
			aSession = HibernateUtil.getSessionFactory().openSession();
			
			if((STATIC.compareToIgnoreCase(binding) == 0) && (SPECIFIC.compareToIgnoreCase(versionToUse) == 0))
				process = AuditDBUtils.getProcessByNameAndVersion(aSession, subProcess, staticVersion);
			else
				process = AuditDBUtils.getLastDefinitionForProcess(aSession, subProcess);
			
			for (Step aStep : process.getSteps()) {
				if(aStep.isIsStart()){
					localpart = aStep.getName();
					namespace = aStep.getNamespace();
				}				
			}

		} catch (Exception e) {
			log.error(e);
		} 
		finally {
			if (aSession != null) {
				aSession.close();
			}
		}
			
		out.setProperty(NAMESPACE, namespace);
		out.setProperty(LOCALPART, localpart);
		out.setProperty(PREFIX, prefix);
		out.setContent(in.getContent());
		return true;
	}

}
