/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.smx.components.restartonfailure;

import java.lang.reflect.Field;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.annotation.PostConstruct;
import javax.jbi.JBIException;
import javax.jbi.management.MBeanNames;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Queue;
import javax.jms.QueueBrowser;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSession;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import javax.naming.Context;
import javax.naming.InitialContext;

import org.apache.servicemix.bean.support.TransformBeanSupport;
import org.apache.servicemix.common.JbiConstants;
import org.apache.servicemix.executors.Executor;
import org.apache.servicemix.executors.ExecutorFactory;
import org.apache.servicemix.jbi.container.JBIContainer;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.apache.servicemix.jbi.listener.MessageExchangeListener;
import org.hibernate.Session;
import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.dbutils.HibernateUtil;
import org.spagic.metadb.dbutils.serialization.CalendarConverter;
import org.spagic.metadb.model.TransitionInstance;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;

/**
 * Service Engine to resend a Message Exchange
 * It receives the id of the JournalInstanceService on the database, containing
 * the Message Exchange to resend.
 */
public class RestartOnFailureComponent extends TransformBeanSupport implements MessageExchangeListener {
	private String jmxServiceUrl = "service:jmx:rmi:///jndi/rmi://localhost:1099/jmxrmi";
	private String jmxUser = "smx";
	private String jmxPsw = "smx";
	private String jmsConnectionFactory = "java:comp/env/jms/SpagicJmsConnectionFactory";
	private String spagicMonitorQueue = "org.spagic.monitor.SMXQueue";
	private long aliveInterval= 120000;//millisecond
	private long emptyQueuePollingInterval= 20000;//millisecond
	
	
	private JBIContainer container = null;
	private Queue destinationQueue = null;
	private QueueConnection queueConnection = null;
	private long restartServer;
	protected AtomicBoolean running = new AtomicBoolean(false);
	
	public void debug(Object o){
		if (logger.isDebugEnabled()){
			logger.debug(o);
		}
	}

	public void setContainer(JBIContainer container) {
		this.container = container;
	}
	
	@PostConstruct
	public void init() throws JBIException {
		try {
			//calculate time (millisecond) of server restart 
			long currentTimeMillis = System.currentTimeMillis();
			
			HashMap<String, String[]> env = new HashMap<String, String[]>();
            env.put("jmx.remote.credentials", new String[] { jmxUser , jmxPsw });
            JMXServiceURL url = new JMXServiceURL(jmxServiceUrl);
            JMXConnector jmxc = JMXConnectorFactory.connect(url, env);
            MBeanServerConnection jmxConnection = jmxc.getMBeanServerConnection();
            long millisFromStartServer = (Long)jmxConnection.getAttribute(new ObjectName("java.lang:type=Runtime"), "Uptime");
            restartServer = currentTimeMillis - millisFromStartServer;
			
			// Retrieving JBI container
			MBeanNames componentContextImpl = getContext().getMBeanNames();
			Field field = componentContextImpl.getClass().getDeclaredField("container");
			field.setAccessible(true);
			JBIContainer container = (JBIContainer) field.get(componentContextImpl);
			setContainer(container);
			logger.info("Retrieved JBI container");
			
			Context ctx = new InitialContext();
			ConnectionFactory connectionFactory = (ConnectionFactory)ctx.lookup(jmsConnectionFactory);
			queueConnection = ((QueueConnectionFactory)connectionFactory).createQueueConnection();
			queueConnection.start();
			
		} catch (NoSuchFieldException nsfe){
			logger.error("Unable to retrieve JBI container", nsfe);
		} catch(IllegalAccessException iae){
			logger.error("Unable to retrieve JBI container", iae);
		} catch (Exception ex){
			logger.error("Unable to retrieve JMX connection ", ex);
		} 
	}
		
	
	@Override
    public NormalizedMessage getInMessage(MessageExchange exchange) throws org.apache.servicemix.jbi.exception.NoInMessageAvailableException  {
    	return super.getInMessage(exchange);
    }


	@Override
	protected boolean transform(MessageExchange exchange, NormalizedMessage in, NormalizedMessage out) throws Exception {
		debug("Received a request to restart one or more exchange");
		if (container == null) {
			logger.error("Container not available: impossible restarting message");
			throw new MessagingException("Container not available: impossible restarting message");
		}
		
		ExecutorFactory factory = container.getExecutorFactory(); 
		Executor executor = factory.createExecutor("component." + getContext().getComponentName());
		executor.execute(new Runnable() {
			                public void run(){
			                    waitSpagicQueueEmptying();
			                }
						});
		
		out.setContent(new StringSource("<RESTART_ON_FALURE_DONE/>"));

		return true;
	}
	
	private void waitSpagicQueueEmptying(){
		QueueSession queueSession = null;
		QueueBrowser queueBrowser = null;
		try{
			//waiting until all messages, sent  in smxMonitorQueue before startServer, are consumed
			queueSession = queueConnection.createQueueSession(false, javax.jms.Session.AUTO_ACKNOWLEDGE);
			destinationQueue = queueSession.createQueue(spagicMonitorQueue);
			String messageSelector = "JMSTimestamp<" + restartServer;
			debug("messageSelector is: " +messageSelector);
			
			queueBrowser = queueSession.createBrowser(destinationQueue, messageSelector);
		
			while (queueBrowser.getEnumeration().hasMoreElements()){
				debug("queueBrowser.getEnumeration() has more elements");
				queueBrowser.close();
				queueSession.close();
				queueConnection.stop();
				Thread.currentThread().sleep(emptyQueuePollingInterval);
				queueConnection.start();
				queueSession = queueConnection.createQueueSession(false, javax.jms.Session.AUTO_ACKNOWLEDGE);
				queueBrowser = queueSession.createBrowser(destinationQueue, messageSelector);
			}
			debug("queue  empty");
			Session aSession = null;
			
			aSession = HibernateUtil.getSessionFactory().openSession();
			Date restartServerDate = new Date(restartServer);
			Date dateFrom = new Date(restartServer - aliveInterval);
			debug("Retrieve all JBI processes in ACTIVE status between " + dateFrom.toString() + " and "+ restartServerDate.toString());
			List<Long> processInstancesIdList = AuditDBUtils.getJBIProcessActiveInstances(aSession, dateFrom,restartServerDate);
			for (Long processInstanceId : processInstancesIdList){
				List<TransitionInstance> transitionList = AuditDBUtils.getTransitionInstancesActive(aSession, processInstanceId);
				for(TransitionInstance transitionInstance : transitionList) {
					MessageExchange deserializedExchange = deserializeExchange(transitionInstance.getTransitionData());
					debug("Deserialized exchange:");
					logger.debug(deserializedExchange);
					
					// Check if the state of the deserialized exchange is not null
					if (deserializedExchange == null) {
						String errorMessage = "Exchange to restart not found on database. Check the monitoring configuration: the saving of exchanges may be disabled";
						logger.error(errorMessage);
					}
					
					// only ACTIVE exchanges should be resent
					if (deserializedExchange.getStatus().equals(ExchangeStatus.ACTIVE)) 
						container.resendExchange(deserializedExchange);
				}

				} 
		} catch(InterruptedException ie){
			logger.error("waitSpagicQueueEmptying:", ie);
		} catch (JMSException jmse){
			logger.error("Unable to retrieve queue", jmse);
		} catch (Exception e) {
	    	logger.error("Fault on restarting activity", e);
	    	// Throw the exception: the super class should send a Fault message or an error,
	    	// according to the MEP
		} finally {
			try {
				if (queueBrowser != null)
					queueBrowser.close();
				if (queueSession != null)
					queueSession.close();
				if (queueConnection != null)
					queueConnection.close();
			}catch (JMSException jmse){
				logger.error("Error closing queue:", jmse);
			}
		}
	}
			
	@Override
	public void onMessageExchange(MessageExchange exchange) {
		 // Skip done exchanges
        if (exchange.getStatus() == ExchangeStatus.DONE) {
            return;
        } else if (exchange.getStatus() == ExchangeStatus.ERROR) {
            return;
        }
        
        try {
            InOnly outExchange = null;
            NormalizedMessage in = getInMessage(exchange);
            NormalizedMessage out;
            if (isInAndOut(exchange)) {
                out = exchange.createMessage();
            } else {
                if (getTarget() == null) {
                    throw new IllegalStateException("An IN-ONLY TransformBean has no Target specified");
                }
                outExchange = getExchangeFactory().createInOnlyExchange();
                getTarget().configureTarget(outExchange, getContext());
                outExchange.setProperty(JbiConstants.SENDER_ENDPOINT, getService() + ":" + getEndpoint());
                String processCorrelationId = (String)exchange.getProperty(JbiConstants.CORRELATION_ID);
                if (processCorrelationId != null) {
                    outExchange.setProperty(JbiConstants.CORRELATION_ID, processCorrelationId);
                }
                out = outExchange.createMessage();
            }
            boolean txSync = exchange.isTransacted() && Boolean.TRUE.equals(exchange.getProperty(JbiConstants.SEND_SYNC));
            copyPropertiesAndAttachments(exchange, in, out);
            exchange.setStatus(ExchangeStatus.DONE);
            getDeliveryChannel().send(exchange);
            if (transform(exchange, in, out)) {
                    outExchange.setMessage(out, "in");
                    if (txSync) {
                        getDeliveryChannel().sendSync(outExchange);
                    } else {
                        getDeliveryChannel().send(outExchange);
                    }
            }
        } catch (Exception e) {
            try {
                fail(exchange, e);
            } catch (Exception e2) {
                logger.warn("Unable to handle error: " + e2, e2);
                if (logger.isDebugEnabled()) {
                    logger.debug("Original error: " + e, e);
                }
            }
        }
	}

	/**
	 * Deserialize a byte stream to rebuild a Message Exchange
	 * @param buf Byte stream
	 * @return MessageExchange
	 */
	private MessageExchange deserializeExchange(String xml) {
		MessageExchange exchange = null;
		XStream xstream = new XStream(new DomDriver());
		xstream.alias("messageExchange", MessageExchange.class);
		xstream.registerConverter(new CalendarConverter());
		if (!(xml.length()>0)) {
			return null;
		}
        
		try {
			// Deserialize from a String
			exchange = (MessageExchange)xstream.fromXML(xml);
		} catch (Exception ex) {
			logger.error("Error serializing exchange " + exchange, ex );
		}
        return exchange;
	}
	
	public void stop() throws Exception {
        try {
            if (queueConnection != null) {
                queueConnection.close();
            }
        } finally {
            queueConnection = null;
        }
    }

	public String getJmxServiceUrl() {
		return jmxServiceUrl;
	}

	public void setJmxServiceUrl(String jmxServiceUrl) {
		this.jmxServiceUrl = jmxServiceUrl;
	}

	public String getJmxUser() {
		return jmxUser;
	}

	public void setJmxUser(String jmxUser) {
		this.jmxUser = jmxUser;
	}

	public String getJmxPsw() {
		return jmxPsw;
	}

	public void setJmxPsw(String jmxPsw) {
		this.jmxPsw = jmxPsw;
	}

	public String getJmsConnectionFactory() {
		return jmsConnectionFactory;
	}

	public void setJmsConnectionFactory(String jmsConnectionFactory) {
		this.jmsConnectionFactory = jmsConnectionFactory;
	}

	public String getSpagicMonitorQueue() {
		return spagicMonitorQueue;
	}

	public void setSpagicMonitorQueue(String spagicMonitorQueue) {
		this.spagicMonitorQueue = spagicMonitorQueue;
	}

	public long getAliveInterval() {
		return aliveInterval;
	}

	public void setAliveInterval(long aliveInterval) {
		this.aliveInterval = aliveInterval;
	}

	public Queue getDestinationQueue() {
		return destinationQueue;
	}

	public void setDestinationQueue(Queue destinationQueue) {
		this.destinationQueue = destinationQueue;
	}
}
